//
//  ZegoRTCEngineManager.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/1/8.
//

#import "ZegoRTCEngineManager.h"
#import "KeyCenter.h"

@interface ZegoRTCEngineManager ()<ZegoEventHandler, ZegoCustomVideoRenderHandler, ZegoAudioDataHandler>

@property (nonatomic, strong) ZegoExpressEngine *expressEngine;

@property (nonatomic, strong) NSMutableArray<StreamInfo *> *streamInfoList;
@property (nonatomic, strong) NSArray <ZegoRoomExtraInfo *> *roomExtraInfoList;
//@property (nonatomic, strong) NSMutableArray< id<RTCEventListener> > *rtcEventListenerList;
@property (nonatomic, weak) id <RTCEventListener> rtcEventListener;
// 房间 ID
@property (nonatomic, copy) NSString *roomID;
// 当前用户 ID
@property (nonatomic, copy) NSString *userID;
// 当前用户名
@property (nonatomic, copy) NSString *userName;
@property (nonatomic, assign) BOOL isInRoom;
@property (nonatomic, assign) BOOL isInit;
@property (nonatomic, assign) int memberCount;

@property (nonatomic, strong) NSMutableArray <ZegoUser *>*rtcUserList;
@end

@implementation ZegoRTCEngineManager

+ (instancetype)sharedInstance {
    static ZegoRTCEngineManager *helper = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        helper = [[ZegoRTCEngineManager alloc] init];
    });
    return helper;
}

- (void)initRtc {
    if (self.isInit) {
        return;
    }
    self.isInit = YES;
    ZegoEngineProfile *profile = [[ZegoEngineProfile alloc] init];
    profile.appID = (int)[KeyCenter appID];
    profile.scenario = ZegoScenarioGeneral;
    ZegoExpressEngine *engine = [ZegoExpressEngine createEngineWithProfile:profile eventHandler:self];
    self.expressEngine = engine;
    NSLog(@"RTC Version is %@",[ZegoExpressEngine getVersion]) ;
}

- (void)unInitRtc:(ZegoDestroyCallback)callBack {
    self.isInit = NO;
    self.expressEngine = nil;
    [ZegoExpressEngine destroyEngine:callBack];
    [self.streamInfoList removeAllObjects];
}

- (void)loginRoom:(NSString *)roomID userID:(nonnull NSString *)userID userName:(nonnull NSString *)userName completion:(nonnull JoinRoomCompletion)completion {
    /// 加入房间
    kLog(@"joinRoom roomID:%@", roomID);
    self.roomID = roomID;
    self.userID = userID;
    self.userName = userName;
    self.isInRoom = YES;
    self.joinRoomCompletion = completion;
    ZegoRoomConfig *roomConfig = [ZegoRoomConfig defaultConfig];
    roomConfig.token = [KeyCenter token];
    roomConfig.isUserStatusNotify = YES;
    
    ZegoUser *user = [ZegoUser userWithUserID:userID
                                     userName:userName];
    [self.expressEngine loginRoom:roomID user:user config:roomConfig callback:completion];
}

- (void)logoutRoom {
    self.publishStreamID = @"";
    if (self.isInRoom) {
        self.rtcEventListener = nil;
        self.roomExtraInfoList = nil;
        [self.streamInfoList removeAllObjects];
        [self stopPublishStream];
        [self.expressEngine logoutRoom];
        self.isInRoom = NO;
    }
    [self.rtcUserList removeAllObjects];
}

- (void)startPublishStream:(BOOL)isHost {
    _publishStreamID = [NSString stringWithFormat:@"%@%@_%@", STREAM_PREFIX, self.roomID, self.userID];
    if(isHost) {
        _publishStreamID = [NSString stringWithFormat:@"%@%@_%@%@", STREAM_PREFIX, self.roomID, self.userID, HOSTFLAG];
    }
    kLog(@"startPublishStream StreamID is %@", _publishStreamID);
    [self.expressEngine startPublishingStream:self.publishStreamID];
    [self.expressEngine enableAudioCaptureDevice:true];
    ZegoAudioFrameParam *audioFrameParam = [[ZegoAudioFrameParam alloc] init];
    audioFrameParam.sampleRate = ZegoAudioSampleRate16K;
    audioFrameParam.channel = ZegoAudioChannelMono;
    [self.expressEngine startAudioDataObserver:ZegoAudioDataCallbackBitMaskCaptured param:audioFrameParam];
    StreamInfo *existStreamInfo = [self getStreamInCurrentListWithStreamID:self.publishStreamID];
    if (![self.streamInfoList containsObject:existStreamInfo]) {
        StreamInfo *streamInfo = [[StreamInfo alloc] init];
        streamInfo.userID = self.userID;
        streamInfo.userName = self.userName;
        streamInfo.streamID = self.publishStreamID;
        [self.streamInfoList insertObject:streamInfo atIndex:0];
    }
}

- (void)stopPublishStream {
    if (self.expressEngine != nil) {
        [self.expressEngine stopPublishingStream];
        [self.expressEngine stopAudioDataObserver];
    }
    StreamInfo *existStreamInfo = [self getStreamInCurrentListWithStreamID:self.publishStreamID];
    [self.streamInfoList removeObject:existStreamInfo];
}

- (void)startPlayingStream:(NSString *)streamID renderView:(UIView *)view {
    kLog(@"startPlayingStream StreamID is %@", streamID);
    if (view != nil) {
        ZegoCanvas *canvas = [ZegoCanvas canvasWithView:view];
        canvas.viewMode = ZegoViewModeAspectFill;
        [self.expressEngine startPlayingStream:streamID canvas:canvas];
    } else {
        [self.expressEngine startPlayingStream:streamID];
    }
}

- (void)stopPlayingStream:(NSString *)streamID {
    [self.expressEngine stopPlayingStream:streamID];
}

- (BOOL)isInRoom {
    return _isInRoom;
}

- (void)startPreview:(UIView *)renderView {
    ZegoCanvas *canvas = [ZegoCanvas canvasWithView:renderView];
    canvas.viewMode = ZegoViewModeAspectFill;
    [self.expressEngine startPreview:canvas];
}

- (NSArray<StreamInfo *> *)getStreamInfoList {
    return _streamInfoList;
}

- (int)getMemberCount {
    return _memberCount;
}


- (void)addRTCEventListener:(id<RTCEventListener>)listener {
    self.rtcEventListener = listener;
}

- (void)removeRTCEventListener:(id<RTCEventListener>)listener {
    self.rtcEventListener = nil;
}

- (void)setRoomExtraInfo:(NSString *)roomID forKey:(NSString *)key value:(NSString *)value {
    [self.expressEngine setRoomExtraInfo:[value modelToJSONString] forKey:key roomID:roomID callback:^(int errorCode) {
        NSLog(@"~~~~~~~~~~~");
    }];
}

- (NSArray<ZegoRoomExtraInfo *> *)getRoomExtraInfoList {
    return _roomExtraInfoList;
}

- (void)muteMicrophone:(BOOL)mute {
    [self.expressEngine mutePublishStreamAudio:mute];
}

- (void)muteSpeaker:(NSString *)userID mute:(BOOL)mute {
    NSArray<StreamInfo *>* streamInfoList = [self getStreamInfoList];
    for (StreamInfo *info in streamInfoList) {
        if([info.userID isEqualToString:userID] && userID.length) {
            [self.expressEngine mutePlayStreamAudio:mute streamID:info.streamID];
        }
    }
}

- (void)initAudioGameConfig {
    ZegoEngineConfig *zegoEngineConfig = [[ZegoEngineConfig alloc] init];
    zegoEngineConfig.advancedConfig = @{@"max_channels" : @"16", @"enable_dtx" : @"true"};
    [ZegoExpressEngine setEngineConfig:zegoEngineConfig];
    
    ZegoAudioConfig *zegoAudioConfig = [[ZegoAudioConfig alloc] init];
    zegoAudioConfig.codecID = ZegoAudioCodecIDLow3;
    zegoAudioConfig.bitrate = ZegoAudioSampleRate48K;
    [self.expressEngine setAudioConfig:zegoAudioConfig];
    
    [self.expressEngine setAudioDeviceMode:ZegoAudioDeviceModeCommunication2];
    [self.expressEngine enableAGC:true];
    [self.expressEngine enableCamera:false];
}

- (NSArray <ZegoUser *>*)getRtcUserList {
    return self.rtcUserList;
}

- (void)onStreamAdd:(NSArray<StreamInfo *> *)streamInfoList {
    [self.streamInfoList addObjectsFromArray:streamInfoList];
    [self.rtcEventListener onStreamAdd:streamInfoList];
}

- (void)onStreamRemove:(NSArray <StreamInfo *> *)streamInfoList {
    for (StreamInfo *streamInfo in streamInfoList) {
        for (StreamInfo *sInfo in self.streamInfoList) {
            if ([sInfo.streamID isEqualToString:streamInfo.streamID]) {
                [self.streamInfoList removeObject:sInfo];
                break;
            }
        }
    }
    [self.rtcEventListener onStreamRemove:streamInfoList];
}


- (void)sendSEIData:(NSData *)data {
    [self.expressEngine sendSEI:data];
}

- (void)onReceiveSEI:(NSData *)data streamID:(NSString *)streamID {
     
}

- (void)onRoomExtraInfoUpdate:(NSString *)roomID arrayList:(NSArray<ZegoRoomExtraInfo *> *)arrayList {
    self.roomExtraInfoList = arrayList;
    [self.rtcEventListener onRoomExtraInfoUpdate:roomID arrayList:arrayList];
}

- (void)onCaptureAudioData:(const unsigned char *)data dataLength:(int)dataLength param:(ZegoAudioFrameParam *)param  {
    
}

- (void)onRoomStateChanged:(RoomStateChangeReason)reason {
    if (reason == Reconnected) {
        if (!ZGStringIsEmpty(self.publishStreamID)) {
            [self startPublishStream:[self.publishStreamID hasSuffix:HOSTFLAG]];
        }
    } else {
        [self logoutRoom];
        [self.rtcEventListener onRoomStateChanged:reason];
    }
}

- (void)onRoomOnlineUserCountUpdate:(NSString *)roomID count:(int)count {
    self.memberCount = count;
    [self.rtcEventListener onRoomOnlineUserCountUpdate:roomID count:count];
}

- (StreamInfo *)getStreamInCurrentListWithStreamID:(NSString *)streamID {
    __block StreamInfo *existStream = nil;
    [self.streamInfoList enumerateObjectsUsingBlock:^(StreamInfo * _Nonnull streamInfo, NSUInteger idx, BOOL * _Nonnull stop) {
        if ([streamInfo.streamID isEqualToString:streamID]) {
            existStream = streamInfo;
            *stop = YES;
        }
    }];
    return existStream;
}

#pragma mark----ZegoAudioDataHandler

- (void)onRoomStateUpdate:(ZegoRoomState)state errorCode:(int)errorCode extendedData:(NSDictionary *)extendedData roomID:(NSString *)roomID {
    kLog(@"onRoomStateUpdate roomID:%@", roomID);
}

- (void)onRoomStateChanged:(ZegoRoomStateChangedReason)reason errorCode:(int)errorCode extendedData:(NSDictionary *)extendedData roomID:(NSString *)roomID {
    kLog(@"onRoomStateChanged roomID：%@, errorCode：%d, reason：%lu, extendedData：%@", roomID, errorCode, (unsigned long)reason, extendedData);
    if (reason == ZegoRoomStateChangedReasonKickOut) {
        [self onRoomStateChanged:KickOut];
    } else if (reason == ZegoRoomStateChangedReasonReconnectFailed) {
        [self onRoomStateChanged:Disconnect];
    } else if (reason == ZegoRoomStateChangedReasonLoginFailed) {
        [self onRoomStateChanged:LoginFailed];
    } else if (reason == ZegoRoomStateChangedReasonReconnected) {
        [self onRoomStateChanged:Reconnected];
    }
}

- (void)onRoomStreamUpdate:(ZegoUpdateType)updateType streamList:(NSArray<ZegoStream *> *)streamList extendedData:(NSDictionary *)extendedData roomID:(NSString *)roomID {
    kLog(@"onRoomStreamUpdate roomID：%@, updateType：%lu, streamList：%@, extendedData：%@", roomID, updateType, streamList, extendedData);
    NSMutableArray<StreamInfo *> *streamInfoList = [NSMutableArray array];
    for (ZegoStream *stream in streamList) {
        StreamInfo *streamInfo = [[StreamInfo alloc] init];
        streamInfo.userID = stream.user.userID;
        streamInfo.userName = stream.user.userName;
        streamInfo.streamID = stream.streamID;
        [streamInfoList addObject:streamInfo];
    }
    if (updateType == ZegoUpdateTypeAdd) {
        [self onStreamAdd:streamInfoList];
    } else {
        [self onStreamRemove:streamInfoList];
    }
    [streamInfoList removeAllObjects];
    streamInfoList = nil;
}

- (void)onRoomStreamExtraInfoUpdate:(NSArray<ZegoStream *> *)streamList roomID:(NSString *)roomID {
    
}

- (void)onRoomExtraInfoUpdate:(NSArray<ZegoRoomExtraInfo *> *)roomExtraInfoList roomID:(NSString *)roomID {
    [self onRoomExtraInfoUpdate:roomID arrayList:roomExtraInfoList];
}

- (void)onPlayerRecvSEI:(NSData *)data streamID:(NSString *)streamID {
    kLog(@"onPlayerRecvSEI data：%@, streamID：%@", data, streamID);
}

- (void)onPublisherQualityUpdate:(ZegoPublishStreamQuality *)quality streamID:(NSString *)streamID {
    
}

- (void)onPlayerQualityUpdate:(ZegoPlayStreamQuality *)quality streamID:(NSString *)streamID {
    
}

- (void)onPlayerStateUpdate:(ZegoPlayerState)state errorCode:(int)errorCode extendedData:(NSDictionary *)extendedData streamID:(NSString *)streamID {
    
}

- (void)onPublisherStateUpdate:(ZegoPublisherState)state errorCode:(int)errorCode extendedData:(NSDictionary *)extendedData streamID:(NSString *)streamID {
   
}

- (void)onRoomOnlineUserCountUpdate:(int)count roomID:(NSString *)roomID {
    [self onRoomOnlineUserCountUpdate:roomID count:count];
}

- (void)onRoomUserUpdate:(ZegoUpdateType)updateType userList:(NSArray<ZegoUser *> *)userList roomID:(NSString *)roomID {
    if(updateType == ZegoUpdateTypeAdd) {
        for (ZegoUser *user in userList) {
            [_rtcUserList addObject:user];
        }
    } else {
        for (ZegoUser *user in userList) {
            for (ZegoUser *rtcUser in _rtcUserList) {
                if([user.userID isEqualToString:rtcUser.userID]) {
                    [_rtcUserList removeObject:rtcUser];
                    break;
                }
            }
        }
    }
    [self.rtcEventListener onRoomUserUpdate:_rtcUserList];
}

- (void)onPublisherStreamEvent:(ZegoStreamEvent)eventID streamID:(NSString *)streamID extraInfo:(NSString *)extraInfo {
    
    NSLog(@"onPublisherStreamEvent is %ld streamID:%@", eventID,streamID);
    if([self.rtcEventListener respondsToSelector:@selector(onRoomPublisherStreamEvent:streamID:extraInfo:)]) {
        [self.rtcEventListener onRoomPublisherStreamEvent:eventID streamID:streamID extraInfo:extraInfo];
    }
}

#pragma mark----ZegoAudioDataHandle

- (void)onCapturedAudioData:(const unsigned char *)data dataLength:(unsigned int)dataLength param:(ZegoAudioFrameParam *)param {
    [self onCapturedAudioData:data dataLength:dataLength param:param];
}


#pragma mark----getter
- (NSMutableArray<StreamInfo *> *)streamInfoList {
    if (_streamInfoList == nil) {
        _streamInfoList = [NSMutableArray array];
    }
    return _streamInfoList;
}

- (NSMutableArray<ZegoUser *> *)rtcUserList {
    if(_rtcUserList == nil) {
        _rtcUserList = [NSMutableArray array];
    }
    return _rtcUserList;
}
@end


@implementation StreamInfo


@end

@implementation RTCUser



@end
